@@ -95,7 +95,6 @@ static int impl_node_enum_params(void *object, int seq,
 				 const struct spa_pod *filter)
 {
 	struct impl *this = object;
-	struct spa_pod *param;
 	struct spa_pod_builder b = { 0 };
 	uint8_t buffer[1024];
 	struct spa_result_node_params result;
@@ -117,25 +116,38 @@ next:
 	switch (id) {
 	case SPA_PARAM_EnumPortConfig:
 	case SPA_PARAM_PortConfig:
+		res = spa_node_enum_params(this->convert, seq, id, start, num, filter);
+		return res;
 	case SPA_PARAM_PropInfo:
 	case SPA_PARAM_Props:
-		return spa_node_enum_params(this->convert, seq, id, start, num, filter);
-
+	{
+		if (result.next < 0x10000) {
+			if ((res = spa_node_enum_params_sync(this->convert,
+					id, &result.next, filter, &result.param, &b)) == 1)
+				break;
+			result.next = 0x10000;
+		}
+		if (result.next >= 0x10000) {
+			result.next &= 0xffff;
+			if ((res = spa_node_enum_params_sync(this->follower,
+					id, &result.next, filter, &result.param, &b)) == 1) {
+				result.next |= 0x10000;
+				break;
+			}
+		}
+		return res;
+	}
 	case SPA_PARAM_EnumFormat:
 	case SPA_PARAM_Format:
 		if ((res = spa_node_port_enum_params_sync(this->follower,
 				this->direction, 0,
-				id, &result.next, filter, &param, &b)) != 1)
-			return res;
-		break;
-
+				id, &result.next, filter, &result.param, &b)) == 1)
+			break;
+		return res;
 	default:
 		return -ENOENT;
 	}
 
-	if (spa_pod_filter(&b, &result.param, param, filter) < 0)
-		goto next;
-
 	spa_node_emit_result(&this->hooks, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
 
 	if (++count != num)
@@ -381,7 +393,7 @@ static int configure_format(struct impl *this, uint32_t flags, const struct spa_
 static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 			       const struct spa_pod *param)
 {
-	int res = 0;
+	int res = 0, res2 = 0;
 	struct impl *this = object;
 	struct spa_audio_info info = { 0 };
 
@@ -415,10 +427,12 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 		break;
 
 	case SPA_PARAM_Props:
-		if (this->target != this->follower) {
-			if ((res = spa_node_set_param(this->target, id, flags, param)) < 0)
-				return res;
-		}
+		if (this->target != this->follower)
+			res = spa_node_set_param(this->target, id, flags, param);
+		res = spa_node_set_param(this->follower, id, flags, param);
+		if (res < 0 && res2 < 0)
+			return res;
+		res = 0;
 		break;
 	default:
 		res = -ENOTSUP;
@@ -622,6 +636,7 @@ static const struct spa_node_events convert_node_events = {
 static void follower_info(void *data, const struct spa_node_info *info)
 {
 	struct impl *this = data;
+	uint32_t i;
 
 	this->async = (info->flags & SPA_NODE_FLAG_ASYNC) != 0;
 
@@ -640,8 +655,23 @@ static void follower_info(void *data, const struct spa_node_info *info)
 	if (info->change_mask & SPA_NODE_CHANGE_MASK_PROPS) {
 		this->info.change_mask |= SPA_NODE_CHANGE_MASK_PROPS;
 		this->info.props = info->props;
-		emit_node_info(this, false);
 	}
+	if (info->change_mask & SPA_NODE_CHANGE_MASK_PARAMS) {
+		for (i = 0; i < info->n_params; i++) {
+			uint32_t idx = SPA_ID_INVALID;
+
+			switch (info->params[i].id) {
+			case SPA_PARAM_Props:
+				idx = 2;
+				break;
+			}
+			if (idx != SPA_ID_INVALID) {
+				this->params[idx] = info->params[i];
+				this->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
+			}
+		}
+	}
+	emit_node_info(this, false);
 }
 
 static void follower_port_info(void *data,
@@ -651,17 +681,19 @@ static void follower_port_info(void *data,
 	struct impl *this = data;
 	uint32_t i;
 
-	for (i = 0; i < info->n_params; i++) {
-		uint32_t idx = SPA_ID_INVALID;
+	if (info->change_mask & SPA_PORT_CHANGE_MASK_PARAMS) {
+		for (i = 0; i < info->n_params; i++) {
+			uint32_t idx = SPA_ID_INVALID;
 
-		switch (info->params[i].id) {
-		case SPA_PARAM_Format:
-			idx = 3;
-			break;
-		}
-		if (idx != SPA_ID_INVALID) {
-			this->params[idx] = info->params[i];
-			this->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
+			switch (info->params[i].id) {
+			case SPA_PARAM_Format:
+				idx = 3;
+				break;
+			}
+			if (idx != SPA_ID_INVALID) {
+				this->params[idx] = info->params[i];
+				this->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
+			}
 		}
 	}
 	if (!this->add_listener)
