@@ -85,8 +85,9 @@ struct port {
 
 	struct spa_list free;
 	struct spa_list ready;
-	uint32_t n_ready;
-	unsigned int buffering:1;
+
+	struct buffer *current_buffer;
+	uint32_t ready_offset;
 };
 
 struct impl {
@@ -111,6 +112,7 @@ struct impl {
 	struct port port;
 
 	unsigned int started:1;
+	unsigned int transport_acquired:1;
 	unsigned int following:1;
 
 	struct spa_source source;
@@ -124,7 +126,8 @@ struct impl {
 
 	uint8_t buffer_read[4096];
 	struct timespec now;
-	uint32_t sample_count;
+	uint64_t sample_count;
+	uint64_t skip_count;
 };
 
 #define NAME "a2dp-source"
@@ -294,8 +297,7 @@ static void reset_buffers(struct port *port)
 
 	spa_list_init(&port->free);
 	spa_list_init(&port->ready);
-	port->n_ready = 0;
-	port->buffering = true;
+	port->current_buffer = NULL;
 
 	for (i = 0; i < port->n_buffers; i++) {
 		struct buffer *b = &port->buffers[i];
@@ -372,14 +374,30 @@ static int32_t decode_data(struct impl *this, uint8_t *src, uint32_t src_size,
 	return dst_size - avail;
 }
 
+static void skip_ready_buffers(struct impl *this)
+{
+	struct port *port = &this->port;
+
+	/* Move all buffers from ready to free */
+	while (!spa_list_is_empty(&port->ready)) {
+		struct buffer *b;
+		b = spa_list_first(&port->ready, struct buffer, link);
+		spa_list_remove(&b->link);
+		spa_list_append(&port->free, &b->link);
+		spa_assert(!b->outstanding);
+		this->skip_count += b->buf->datas[0].chunk->size / port->frame_size;
+	}
+}
+
 static void a2dp_on_ready_read(struct spa_source *source)
 {
 	struct impl *this = source->data;
 	struct port *port = &this->port;
 	struct spa_io_buffers *io = port->io;
-	int32_t size_read, decoded;
+	int32_t size_read, decoded, avail;
 	struct spa_data *datas;
 	struct buffer *buffer;
+	uint32_t min_data;
 	uint8_t read_decoded[4096];
 
 	/* make sure the source is an input */
@@ -403,7 +421,7 @@ static void a2dp_on_ready_read(struct spa_source *source)
 		spa_log_error(this->log, "failed to read data: %s", spa_strerror(size_read));
 		goto stop;
 	}
-	spa_log_debug(this->log, "read socket data %d", size_read);
+	spa_log_trace(this->log, "read socket data %d", size_read);
 
 	/* decode */
 	decoded = decode_data(this, this->buffer_read, size_read,
@@ -415,48 +433,75 @@ static void a2dp_on_ready_read(struct spa_source *source)
 	if (decoded == 0)
 		return;
 
-	spa_log_debug(this->log, "decoded socket data %d", decoded);
+	spa_log_trace(this->log, "decoded socket data %d", decoded);
 
 	/* discard when not started */
 	if (!this->started)
 		return;
 
 	/* get buffer */
-	if (spa_list_is_empty(&port->free)) {
-		spa_log_warn(this->log, "no buffer available");
-		return;
-	}
-	buffer = spa_list_first(&port->free, struct buffer, link);
-	spa_list_remove(&buffer->link);
-	spa_log_debug(this->log, "dequeue %d", buffer->id);
+	if (!port->current_buffer) {
+		if (spa_list_is_empty(&port->free)) {
+			/* xrun, skip ahead */
+			skip_ready_buffers(this);
+			this->skip_count += decoded / port->frame_size;
+			this->sample_count += decoded / port->frame_size;
+			return;
+		}
+		if (this->skip_count > 0) {
+			spa_log_info(this->log, NAME " %p: xrun, skipped %"PRIu64" usec",
+			             this, (uint64_t)(this->skip_count * SPA_USEC_PER_SEC / port->current_format.info.raw.rate));
+			this->skip_count = 0;
+		}
 
-	if (buffer->h) {
-		buffer->h->seq = this->sample_count;
-		buffer->h->pts = SPA_TIMESPEC_TO_NSEC(&this->now);
-		buffer->h->dts_offset = 0;
+		buffer = spa_list_first(&port->free, struct buffer, link);
+		spa_list_remove(&buffer->link);
+
+		port->current_buffer = buffer;
+		port->ready_offset = 0;
+		spa_log_trace(this->log, "dequeue %d", buffer->id);
+
+		if (buffer->h) {
+			buffer->h->seq = this->sample_count;
+			buffer->h->pts = SPA_TIMESPEC_TO_NSEC(&this->now);
+			buffer->h->dts_offset = 0;
+		}
+	} else {
+		buffer = port->current_buffer;
 	}
 	datas = buffer->buf->datas;
 
 	/* copy data into buffer */
-	memcpy ((uint8_t *)datas[0].data, read_decoded, decoded);
+	avail = SPA_MIN(decoded, (int32_t)(datas[0].maxsize - port->ready_offset));
+	if (avail < decoded)
+		spa_log_warn(this->log, NAME ": buffer too small (%d > %d)", decoded, avail);
+	memcpy ((uint8_t *)datas[0].data + port->ready_offset, read_decoded, avail);
+	port->ready_offset += avail;
+	this->sample_count += decoded / port->frame_size;
 
-	/* send buffer */
-	datas[0].chunk->offset = 0;
-	datas[0].chunk->size = decoded;
-	datas[0].chunk->stride = port->frame_size;
+	/* send buffer if full */
+	min_data = SPA_MIN(this->props.min_latency * port->frame_size, datas[0].maxsize / 2);
+	if (port->ready_offset >= min_data) {
+		uint64_t sample_count;
 
-	this->sample_count += datas[0].chunk->size / port->frame_size;
+		datas[0].chunk->offset = 0;
+		datas[0].chunk->size = port->ready_offset;
+		datas[0].chunk->stride = port->frame_size;
 
-	spa_log_debug(this->log, "queue %d", buffer->id);
-	spa_list_append(&port->ready, &buffer->link);
-	port->n_ready++;
+		sample_count = datas[0].chunk->size / port->frame_size;
 
-	if (!this->following && this->clock) {
-		this->clock->nsec = SPA_TIMESPEC_TO_NSEC(&this->now);
-		this->clock->position = this->sample_count;
-		this->clock->delay = 0;
-		this->clock->rate_diff = 1.0f;
-		this->clock->next_nsec = this->clock->nsec;
+		spa_log_trace(this->log, "queue %d", buffer->id);
+		spa_list_append(&port->ready, &buffer->link);
+		port->current_buffer = NULL;
+
+		if (!this->following && this->clock) {
+			this->clock->nsec = SPA_TIMESPEC_TO_NSEC(&this->now);
+			this->clock->duration = sample_count * this->clock->rate.denom / port->current_format.info.raw.rate;
+			this->clock->position = this->sample_count * this->clock->rate.denom / port->current_format.info.raw.rate;
+			this->clock->delay = 0;
+			this->clock->rate_diff = 1.0f;
+			this->clock->next_nsec = this->clock->nsec + (uint64_t)sample_count * SPA_NSEC_PER_SEC / port->current_format.info.raw.rate;
+		}
 	}
 
 	/* done if there are no buffers ready */
@@ -475,8 +520,6 @@ static void a2dp_on_ready_read(struct spa_source *source)
 
 		b = spa_list_first(&port->ready, struct buffer, link);
 		spa_list_remove(&b->link);
-		if (--port->n_ready == 0)
-			port->buffering = true;
 		b->outstanding = true;
 
 		io->buffer_id = b->id;
@@ -497,11 +540,16 @@ static int transport_start(struct impl *this)
 	int res, val;
 	struct port *port = &this->port;
 
+	if (this->transport_acquired)
+		return 0;
+
 	spa_log_debug(this->log, NAME" %p: transport %p acquire", this,
 			this->transport);
 	if ((res = spa_bt_transport_acquire(this->transport, false)) < 0)
 		return res;
 
+	this->transport_acquired = true;
+
 	this->codec_data = this->codec->init(this->codec, 0,
 			this->transport->configuration,
 			this->transport->configuration_len,
@@ -538,6 +586,7 @@ static int transport_start(struct impl *this)
 	spa_loop_add_source(this->data_loop, &this->source);
 
 	this->sample_count = 0;
+	this->skip_count = 0;
 
 	return 0;
 }
@@ -551,8 +600,8 @@ static int do_start(struct impl *this)
 
 	this->following = is_following(this);
 
-	spa_log_debug(this->log, NAME" %p: start state:%d",
-			this, this->transport->state);
+	spa_log_debug(this->log, NAME" %p: start state:%d following:%d",
+			this, this->transport->state, this->following);
 
 	spa_return_val_if_fail(this->transport != NULL, -EIO);
 
@@ -579,6 +628,28 @@ static int do_remove_source(struct spa_loop *loop,
 	return 0;
 }
 
+static int transport_stop(struct impl *this)
+{
+	int res;
+
+	spa_log_debug(this->log, NAME" %p: transport stop", this);
+
+	spa_loop_invoke(this->data_loop, do_remove_source, 0, NULL, 0, true, this);
+
+	if (this->transport && this->transport_acquired)
+		res = spa_bt_transport_release(this->transport);
+	else
+		res = 0;
+
+	this->transport_acquired = false;
+
+	if (this->codec_data)
+		this->codec->deinit(this->codec_data);
+	this->codec_data = NULL;
+
+	return res;
+}
+
 static int do_stop(struct impl *this)
 {
 	int res;
@@ -588,19 +659,10 @@ static int do_stop(struct impl *this)
 
 	spa_log_debug(this->log, NAME" %p: stop", this);
 
-	spa_loop_invoke(this->data_loop, do_remove_source, 0, NULL, 0, true, this);
+	res = transport_stop(this);
 
 	this->started = false;
 
-	if (this->transport)
-		res = spa_bt_transport_release(this->transport);
-	else
-		res = 0;
-
-	if (this->codec_data)
-		this->codec->deinit(this->codec_data);
-	this->codec_data = NULL;
-
 	return res;
 }
 
@@ -636,16 +698,16 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 	return 0;
 }
 
-static const struct spa_dict_item node_info_items[] = {
-	{ SPA_KEY_DEVICE_API, "bluez5" },
-	{ SPA_KEY_MEDIA_CLASS, "Stream/Output/Audio" },
-	{ SPA_KEY_NODE_DRIVER, "true" },
-	{ SPA_KEY_NODE_LATENCY, "512/48000" },
-	{ SPA_KEY_NODE_PAUSE_ON_IDLE, "false" },
-};
-
 static void emit_node_info(struct impl *this, bool full)
 {
+	struct spa_dict_item node_info_items[] = {
+		{ SPA_KEY_DEVICE_API, "bluez5" },
+		{ SPA_KEY_MEDIA_CLASS, "Stream/Output/Audio" },
+		{ SPA_KEY_NODE_LATENCY, SPA_STRINGIFY(MIN_LATENCY)"/48000" },
+		{ "media.name", ((this->transport && this->transport->device->name) ?
+		                 this->transport->device->name : "A2DP") },
+	};
+
 	if (full)
 		this->info.change_mask = this->info_all;
 	if (this->info.change_mask) {
@@ -842,10 +904,9 @@ static int clear_buffers(struct impl *this, struct port *port)
 	if (port->n_buffers > 0) {
 		spa_list_init(&port->free);
 		spa_list_init(&port->ready);
-		port->n_ready = 0;
-		port->buffering = true;
 		port->n_buffers = 0;
 	}
+	port->current_buffer = NULL;
 	return 0;
 }
 
@@ -1039,7 +1100,7 @@ static int impl_node_process(void *object)
 	io = port->io;
 	spa_return_val_if_fail(io != NULL, -EIO);
 
-	spa_log_debug(this->log, "%p status:%d %d", this, io->status, port->n_ready);
+	spa_log_trace(this->log, "%p status:%d", this, io->status);
 
 	/* Return if we already have a buffer */
 	if (io->status == SPA_STATUS_HAVE_DATA)
@@ -1054,16 +1115,10 @@ static int impl_node_process(void *object)
 	/* Return if there are no buffers ready to be processed */
 	if (spa_list_is_empty(&port->ready))
 		return SPA_STATUS_OK;
-	if (port->buffering && port->n_ready < 4)
-		return SPA_STATUS_OK;
-
-	port->buffering = false;
 
 	/* Get the new buffer from the ready list */
 	buffer = spa_list_first(&port->ready, struct buffer, link);
 	spa_list_remove(&buffer->link);
-	if (--port->n_ready == 0)
-		port->buffering = true;
 	buffer->outstanding = true;
 
 	/* Set the new buffer in IO */
@@ -1102,6 +1157,7 @@ static int do_transport_destroy(struct spa_loop *loop,
 {
 	struct impl *this = user_data;
 	this->transport = NULL;
+	this->transport_acquired = false;
 	return 0;
 }
 
@@ -1121,6 +1177,8 @@ static void transport_state_changed(void *data, enum spa_bt_transport_state old,
 
 	if (state >= SPA_BT_TRANSPORT_STATE_PENDING && old < SPA_BT_TRANSPORT_STATE_PENDING)
 		transport_start(this);
+	else if (state < SPA_BT_TRANSPORT_STATE_PENDING && old >= SPA_BT_TRANSPORT_STATE_PENDING)
+		transport_stop(this);
 }
 
 static const struct spa_bt_transport_events transport_events = {
