@@ -476,7 +476,7 @@ static int device_add(struct spa_bt_monitor *monitor, struct spa_bt_device *devi
 	struct spa_dict_item items[20];
 	uint32_t n_items = 0;
 
-	if (device->added)
+	if (device->connected_profiles == 0 || device->added)
 		return 0;
 
 	info = SPA_DEVICE_OBJECT_INFO_INIT();
@@ -601,8 +601,7 @@ int spa_bt_device_check_profiles(struct spa_bt_device *device, bool force)
 			device_stop_timer(device);
 			device_remove(monitor, device);
 		}
-	}
-	else if (force || (device->profiles & connected_profiles) == device->profiles) {
+	} else if (force || (device->profiles & connected_profiles) == device->profiles) {
 		device_stop_timer(device);
 		device_add(monitor, device);
 	} else {
@@ -613,6 +612,8 @@ int spa_bt_device_check_profiles(struct spa_bt_device *device, bool force)
 
 static void device_set_connected(struct spa_bt_device *device, int connected)
 {
+	struct spa_bt_monitor *monitor = device->monitor;
+
 	if (device->connected && !connected)
 		device->connected_profiles = 0;
 
@@ -620,8 +621,11 @@ static void device_set_connected(struct spa_bt_device *device, int connected)
 
 	if (connected)
 		spa_bt_device_check_profiles(device, false);
-	else
+	else {
 		device_stop_timer(device);
+		if (device->added)
+			device_remove(monitor, device);
+	}
 }
 
 int spa_bt_device_connect_profile(struct spa_bt_device *device, enum spa_bt_profile profile)
@@ -1422,6 +1426,7 @@ static int transport_release(void *data)
 	struct spa_bt_monitor *monitor = transport->monitor;
 	DBusMessage *m, *r;
 	DBusError err;
+	bool is_idle = (transport->state == SPA_BT_TRANSPORT_STATE_IDLE);
 
 	spa_log_debug(monitor->log, NAME": transport %p: Release %s",
 			transport, transport->path);
@@ -1446,8 +1451,18 @@ static int transport_release(void *data)
 		dbus_message_unref(r);
 
 	if (dbus_error_is_set(&err)) {
-		spa_log_error(monitor->log, "Failed to release transport %s: %s",
-				transport->path, err.message);
+		if (is_idle) {
+			/* XXX: The fd always needs to be closed. However, Release()
+			 * XXX: apparently doesn't need to be called on idle transports
+			 * XXX: and fails. We call it just to be sure (e.g. in case
+			 * XXX: there's a race with updating the property), but tone down the error.
+			 */
+			spa_log_debug(monitor->log, "Failed to release idle transport %s: %s",
+			              transport->path, err.message);
+		} else {
+			spa_log_error(monitor->log, "Failed to release transport %s: %s",
+			              transport->path, err.message);
+		}
 		dbus_error_free(&err);
 	}
 	else
@@ -1698,7 +1713,7 @@ static int a2dp_codec_switch_cmp(const void *a, const void *b)
 {
 	struct spa_bt_a2dp_codec_switch *sw = a2dp_codec_switch_cmp_sw;
 	const struct a2dp_codec *codec = *sw->codec_iter;
-	const char *path1 = a, *path2 = b;
+	const char *path1 = *(char **)a, *path2 = *(char **)b;
 	struct spa_bt_remote_endpoint *ep1, *ep2;
 
 	ep1 = device_remote_endpoint_find(sw->device, path1);
@@ -2200,6 +2215,7 @@ static void append_a2dp_object(DBusMessageIter *iter, const char *endpoint,
 	char* str;
 	const char *interface_name = BLUEZ_MEDIA_ENDPOINT_INTERFACE;
 	DBusMessageIter object, array, entry, dict;
+	dbus_bool_t delay_reporting;
 
 	dbus_message_iter_open_container(iter, DBUS_TYPE_DICT_ENTRY, NULL, &object);
 	dbus_message_iter_append_basic(&object, DBUS_TYPE_OBJECT_PATH, &endpoint);
@@ -2217,6 +2233,11 @@ static void append_a2dp_object(DBusMessageIter *iter, const char *endpoint,
 	append_basic_variant_dict_entry(&dict, DBUS_TYPE_STRING, &str, DBUS_TYPE_BYTE, "y", &codec_id);
 	str = "Capabilities";
 	append_basic_array_variant_dict_entry(&dict, DBUS_TYPE_STRING, &str, "ay", "y", DBUS_TYPE_BYTE, caps, caps_size);
+	if (spa_bt_profile_from_uuid(uuid) & SPA_BT_PROFILE_A2DP_SOURCE) {
+		str = "DelayReporting";
+		delay_reporting = TRUE;
+		append_basic_variant_dict_entry(&dict, DBUS_TYPE_STRING, &str, DBUS_TYPE_BOOLEAN, "b", &delay_reporting);
+	}
 
 	dbus_message_iter_close_container(&entry, &dict);
 	dbus_message_iter_close_container(&array, &entry);
@@ -2919,7 +2940,7 @@ static int parse_codec_array(struct spa_bt_monitor *this, const struct spa_dict
 	if (codecs == NULL)
 		return -ENOMEM;
 
-	if ((str = spa_dict_lookup(info, "bluez5.codecs")) == NULL)
+	if (info == NULL || (str = spa_dict_lookup(info, "bluez5.codecs")) == NULL)
 		goto fallback;
 
 	spa_json_init(&it, str, strlen(str));
