@@ -118,6 +118,19 @@ static void init_node(struct impl *this, struct node *node, uint32_t id)
 		node->volumes[i] = 1.0;
 }
 
+static const char *get_codec_name(struct spa_bt_transport *t)
+{
+	if (t->a2dp_codec != NULL)
+		return t->a2dp_codec->name;
+	switch (t->codec) {
+	case HFP_AUDIO_CODEC_MSBC:
+		return "mSBC";
+	case HFP_AUDIO_CODEC_CVSD:
+		return "CVSD";
+	}
+	return "unknown";
+}
+
 static void emit_node(struct impl *this, struct spa_bt_transport *t,
 		uint32_t id, const char *factory_name)
 {
@@ -129,8 +142,7 @@ static void emit_node(struct impl *this, struct spa_bt_transport *t,
 	snprintf(transport, sizeof(transport), "pointer:%p", t);
 	items[0] = SPA_DICT_ITEM_INIT(SPA_KEY_API_BLUEZ5_TRANSPORT, transport);
 	items[1] = SPA_DICT_ITEM_INIT(SPA_KEY_API_BLUEZ5_PROFILE, spa_bt_profile_name(t->profile));
-	items[2] = SPA_DICT_ITEM_INIT(SPA_KEY_API_BLUEZ5_CODEC,
-			t->a2dp_codec ? t->a2dp_codec->name : "unknown");
+	items[2] = SPA_DICT_ITEM_INIT(SPA_KEY_API_BLUEZ5_CODEC, get_codec_name(t));
 	snprintf(str_id, sizeof(str_id), "%d", id);
 	items[3] = SPA_DICT_ITEM_INIT("card.profile.device", str_id);
 	items[4] = SPA_DICT_ITEM_INIT(SPA_KEY_API_BLUEZ5_ADDRESS, device->address);
@@ -261,11 +273,15 @@ static int set_profile(struct impl *this, uint32_t profile, const struct a2dp_co
 	this->profile = profile;
 	this->selected_a2dp_codec = a2dp_codec;
 
-	if (profile == 1) {
+	/*
+	 * A2DP: ensure there's a transport with the selected codec (NULL means any).
+	 * Don't try to switch codecs when the device is in the A2DP source role, since
+	 * devices do not appear to like that.
+	 */
+	if (profile == 1 && !(this->bt_dev->connected_profiles & SPA_BT_PROFILE_A2DP_SOURCE)) {
 		int ret;
 		const struct a2dp_codec *codec_list[2], **codecs;
 
-		/* A2DP: ensure there's a transport with the selected codec (NULL means any) */
 		if (a2dp_codec == NULL) {
 			codecs = a2dp_codecs;
 		} else {
@@ -464,6 +480,10 @@ static uint32_t get_profile_from_index(struct impl *this, uint32_t index, const
 	if (index < 3)
 		return index;
 
+	/* A2DP sources don't have codec profiles (device chooses it) */
+	if (this->bt_dev->connected_profiles & SPA_BT_PROFILE_A2DP_SOURCE)
+		return SPA_ID_INVALID;
+
 	if (index - 3 < this->supported_codec_count) {
 		*codec = this->supported_codecs[index - 3];
 		return 1;
@@ -482,6 +502,10 @@ static uint32_t get_index_from_profile(struct impl *this, uint32_t profile, cons
 	if (codec == NULL)
 		return 1;
 
+	/* A2DP sources don't have codec profiles (device chooses it) */
+	if (this->bt_dev->connected_profiles & SPA_BT_PROFILE_A2DP_SOURCE)
+		return SPA_ID_INVALID;
+
 	for (i = 0; i < this->supported_codec_count; ++i) {
 		if (this->supported_codecs[i] == codec)
 			return 3 + i;
@@ -490,6 +514,41 @@ static uint32_t get_index_from_profile(struct impl *this, uint32_t profile, cons
 	return SPA_ID_INVALID;
 }
 
+static void set_initial_profile(struct impl *this)
+{
+	struct spa_bt_transport *t;
+	int i;
+
+	/* Prefer A2DP, then HFP, then null */
+
+	for (i = SPA_BT_PROFILE_A2DP_SINK; i <= SPA_BT_PROFILE_A2DP_SOURCE; i <<= 1) {
+		if (!(this->bt_dev->connected_profiles & i))
+			continue;
+
+		t = find_transport(this, i, NULL);
+		if (t) {
+			this->profile = 1;
+			this->selected_a2dp_codec = t->a2dp_codec;
+			return;
+		}
+	}
+
+	for (i = SPA_BT_PROFILE_HSP_HS; i <= SPA_BT_PROFILE_HFP_AG; i <<= 1) {
+		if (!(this->bt_dev->connected_profiles & i))
+			continue;
+
+		t = find_transport(this, i, NULL);
+		if (t) {
+			this->profile = 2;
+			this->selected_a2dp_codec = NULL;
+			return;
+		}
+	}
+
+	this->profile = 0;
+	this->selected_a2dp_codec = NULL;
+}
+
 static struct spa_pod *build_profile(struct impl *this, struct spa_pod_builder *b,
 		uint32_t id, uint32_t index, uint32_t profile_index, const struct a2dp_codec *codec)
 {
@@ -1121,6 +1180,8 @@ impl_init(const struct spa_handle_factory *factory,
 
 	spa_bt_device_add_listener(this->bt_dev, &this->bt_dev_listener, &bt_dev_events, this);
 
+	set_initial_profile(this);
+
 	return 0;
 }
 
