@@ -69,21 +69,17 @@ struct device {
 
 	uint32_t id;
 	struct impl *impl;
+	char *name;
 	char *key;
 
 	struct spa_hook listener;
 
 	unsigned int restored:1;
 
+	uint32_t saved_profile;
 	uint32_t active_profile;
 };
 
-struct find_data {
-	struct impl *impl;
-	const char *name;
-	uint32_t id;
-};
-
 static void remove_idle_timeout(struct impl *impl)
 {
 	struct pw_loop *main_loop = pw_context_get_main_loop(impl->context);
@@ -118,46 +114,109 @@ static void add_idle_timeout(struct impl *impl)
 	pw_loop_update_timer(main_loop, impl->idle_timeout, &value, NULL, false);
 }
 
-static uint32_t find_profile_id(struct device *dev, const char *name)
+struct profile {
+	struct sm_param *p;
+	uint32_t index;
+	const char *name;
+	uint32_t prio;
+	uint32_t available;
+};
+
+static int parse_profile(struct sm_param *p, struct profile *pr)
+{
+	pr->p = p;
+	pr->prio = 0;
+	pr->available = SPA_PARAM_AVAILABILITY_unknown;
+	return spa_pod_parse_object(p->param,
+			SPA_TYPE_OBJECT_ParamProfile, NULL,
+			SPA_PARAM_PROFILE_index, SPA_POD_Int(&pr->index),
+			SPA_PARAM_PROFILE_name,  SPA_POD_String(&pr->name),
+			SPA_PARAM_PROFILE_priority,  SPA_POD_OPT_Int(&pr->prio),
+			SPA_PARAM_PROFILE_available,  SPA_POD_OPT_Id(&pr->available));
+}
+
+static int find_profile(struct device *dev, uint32_t index, struct profile *pr)
 {
 	struct sm_param *p;
 	spa_list_for_each(p, &dev->obj->param_list, link) {
-		const char *n;
-		uint32_t id;
-
 		if (p->id != SPA_PARAM_EnumProfile ||
-		    spa_pod_parse_object(p->param,
-				SPA_TYPE_OBJECT_ParamProfile, NULL,
-				SPA_PARAM_PROFILE_index, SPA_POD_Int(&id),
-				SPA_PARAM_PROFILE_name,  SPA_POD_String(&n)) < 0) {
+		    parse_profile(p, pr) < 0)
 			continue;
-		}
-		if (strcmp(n, name) == 0)
-			return id;
+		if (index == pr->index)
+			return 0;
 	}
-	return SPA_ID_INVALID;
+	return -ENOENT;
 }
 
-static int restore_profile(struct device *dev)
+static int find_current_profile(struct device *dev, struct profile *pr)
+{
+	struct sm_param *p;
+	spa_list_for_each(p, &dev->obj->param_list, link) {
+		if (p->id == SPA_PARAM_Profile &&
+		    parse_profile(p, pr) >= 0)
+			return 0;
+	}
+	return -ENOENT;
+}
+
+static int find_best_profile(struct device *dev, struct profile *pr)
+{
+	struct sm_param *p;
+	struct profile best, best_avail, best_unk, off;
+
+	spa_zero(best);
+	spa_zero(best_avail);
+	spa_zero(best_unk);
+	spa_zero(off);
+
+	spa_list_for_each(p, &dev->obj->param_list, link) {
+		struct profile t;
+
+		if (p->id != SPA_PARAM_EnumProfile ||
+		    parse_profile(p, &t) < 0)
+			continue;
+
+		if (t.name && strcmp(t.name, "off") == 0) {
+			off = t;
+		}
+		else if (t.available == SPA_PARAM_AVAILABILITY_yes) {
+			if (best_avail.name == NULL || t.prio > best_avail.prio)
+				best_avail = t;
+		}
+		else if (t.available != SPA_PARAM_AVAILABILITY_no) {
+			if (best_unk.name == NULL || t.prio > best_unk.prio)
+				best_unk = t;
+		}
+	}
+	best = best_avail;
+	if (best.name == NULL)
+		best = best_unk;
+	if (best.name == NULL)
+		best = off;
+	if (best.name == NULL)
+		return -ENOENT;
+	*pr = best;
+	return 0;
+}
+
+static int find_saved_profile(struct device *dev, struct profile *pr)
 {
 	struct spa_json it[2];
 	struct impl *impl = dev->impl;
 	const char *json, *value;
-	int len;
-	uint32_t index = SPA_ID_INVALID;
-	char buf[1024], name[1024] = "\0";
-	struct spa_pod_builder b = SPA_POD_BUILDER_INIT(buf, sizeof(buf));
+	char name[1024] = "\0", key[128];
+	struct sm_param *p;
 
 	json = pw_properties_get(impl->properties, dev->key);
 	if (json == NULL)
-		return -ENOENT;
+		return -ENODEV;
 
 	spa_json_init(&it[0], json, strlen(json));
 	if (spa_json_enter_object(&it[0], &it[1]) <= 0)
                 return -EINVAL;
 
-	while ((len = spa_json_next(&it[1], &value)) > 0) {
-		if (strncmp(value, "\"name\"", len) == 0) {
+	while (spa_json_get_string(&it[1], key, sizeof(key)-1) > 0) {
+		if (strcmp(key, "name") == 0) {
 			if (spa_json_get_string(&it[1], name, sizeof(name)) <= 0)
                                 continue;
 		} else {
@@ -165,46 +224,119 @@ static int restore_profile(struct device *dev)
                                 break;
 		}
 	}
-	pw_log_debug("device %d: find profile '%s'", dev->id, name);
-	index = find_profile_id(dev, name);
-	if (index == SPA_ID_INVALID)
-		return -ENOENT;
+	pw_log_debug("device '%s': find profile '%s'", dev->name, name);
+
+	spa_list_for_each(p, &dev->obj->param_list, link) {
+		if (p->id != SPA_PARAM_EnumProfile ||
+		    parse_profile(p, pr) < 0)
+			continue;
+
+		if (strcmp(pr->name, name) == 0)
+			return 0;
+	}
+	return -ENOENT;
+}
+
+static int set_profile(struct device *dev, struct profile *pr)
+{
+	char buf[1024];
+	struct spa_pod_builder b = SPA_POD_BUILDER_INIT(buf, sizeof(buf));
+
+	if (dev->active_profile == pr->index)
+		return 0;
 
-	pw_log_info("device %d: restore profile '%s' index %d", dev->id, name, index);
 	pw_device_set_param((struct pw_device*)dev->obj->obj.proxy,
 			SPA_PARAM_Profile, 0,
 			spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_ParamProfile, SPA_PARAM_Profile,
-				SPA_PARAM_PROFILE_index, SPA_POD_Int(index)));
-	dev->active_profile = index;
+				SPA_PARAM_PROFILE_index, SPA_POD_Int(pr->index)));
+
+	dev->active_profile = pr->index;
+
 	return 0;
 }
 
-static int handle_profile(struct device *dev, struct sm_param *p)
+static int handle_profile(struct device *dev)
 {
 	struct impl *impl = dev->impl;
-	uint32_t index;
+	struct profile pr;
 	int res;
 
-	if (!dev->restored) {
-		restore_profile(dev);
-		dev->restored = true;
-	} else {
-		const char *name;
-		if ((res = spa_pod_parse_object(p->param,
-				SPA_TYPE_OBJECT_ParamProfile, NULL,
-				SPA_PARAM_PROFILE_index, SPA_POD_Int(&index),
-				SPA_PARAM_PROFILE_name, SPA_POD_String(&name))) < 0) {
-			pw_log_warn("device %d: can't parse profile: %s", dev->id, spa_strerror(res));
-			return res;
+	if (dev->active_profile != SPA_ID_INVALID) {
+		/* we have a profile, find it */
+		res = find_profile(dev, dev->active_profile, &pr);
+		if (res < 0 || pr.available == SPA_PARAM_AVAILABILITY_no) {
+			/* we had a profile but it is now gone or not available,
+			 * try to restore a new profile */
+			if (res < 0)
+				pw_log_info("device '%s': active profile index %d removed",
+					dev->name, dev->active_profile);
+			else
+				pw_log_info("device '%s': active profile '%s' unavailable",
+					dev->name, pr.name);
+
+			dev->restored = false;
+		} else if (dev->saved_profile && pr.index != dev->saved_profile) {
+			struct profile saved;
+			/* we had a saved profile but we're on some other profile,
+			 * see if the saved profile is available now and switch to it */
+			res = find_saved_profile(dev, &saved);
+			if (res >= 0 && saved.available != SPA_PARAM_AVAILABILITY_no) {
+				pw_log_info("device '%s': restore saved profile '%s' index %d",
+						dev->name, saved.name, saved.index);
+				if (set_profile(dev, &saved) >= 0)
+					dev->restored = true;
+			}
 		}
-		if (dev->active_profile == index)
+	}
+	if (!dev->restored) {
+		/* first try to restore our saved profile */
+		res = find_saved_profile(dev, &pr);
+		if (res < 0 || pr.available == SPA_PARAM_AVAILABILITY_no) {
+			/* no saved profile found or it is not available */
+			if (res < 0)
+				pw_log_info("device '%s': no saved profile: %s",
+					dev->name, spa_strerror(res));
+			else
+				pw_log_info("device '%s': saved profile '%s' unavailable",
+					dev->name, pr.name);
+
+			/* try to find the next best profile */
+			res = find_best_profile(dev, &pr);
+			if (res < 0)
+				pw_log_info("device '%s': can't find best profile: %s",
+						dev->name, spa_strerror(res));
+			else
+				pw_log_info("device '%s': found best profile '%s'",
+						dev->name, pr.name);
+		} else {
+			pw_log_info("device '%s': found saved profile '%s'",
+						dev->name, pr.name);
+			dev->saved_profile = pr.index;
+		}
+		if (res >= 0) {
+			pw_log_info("device '%s': restore profile '%s' index %d",
+					dev->name, pr.name, pr.index);
+			if (set_profile(dev, &pr) >= 0)
+				dev->restored = true;
+		} else {
+			pw_log_warn("device '%s': can't restore profile", dev->name);
+		}
+	} else {
+		if ((res = find_current_profile(dev, &pr)) < 0)
+			return res;
+
+		if (dev->active_profile == pr.index)
 			return 0;
 
-		dev->active_profile = index;
-		pw_log_debug("device %d: current profile %d %s", dev->id, index, name);
-		pw_properties_setf(impl->properties, dev->key, "{ \"name\": \"%s\" }", name);
-		add_idle_timeout(impl);
+		/* we get here when had configured a profile but something
+		 * else changed it, in that case, save it. */
+		dev->active_profile = pr.index;
+		dev->saved_profile = pr.index;
+		if (pw_properties_setf(impl->properties, dev->key, "{ \"name\": \"%s\" }", pr.name)) {
+			pw_log_info("device '%s': active profile changed to '%s'", dev->name, pr.name);
+			add_idle_timeout(impl);
+		}
 	}
 	return 0;
 }
@@ -213,27 +345,18 @@ static void object_update(void *data)
 {
 	struct device *dev = data;
 	struct impl *impl = dev->impl;
+	const char *str;
 
 	pw_log_debug(NAME" %p: device %p %08x/%08x", impl, dev,
 			dev->obj->obj.changed, dev->obj->obj.avail);
 
-	if (dev->obj->obj.changed & SM_NODE_CHANGE_MASK_PARAMS) {
-		struct sm_param *p;
-		spa_list_for_each(p, &dev->obj->param_list, link) {
-			if (pw_log_level_enabled(SPA_LOG_LEVEL_DEBUG))
-				spa_debug_pod(2, NULL, p->param);
+	if (dev->obj->info && dev->obj->info->props &&
+	    (str = spa_dict_lookup(dev->obj->info->props, PW_KEY_DEVICE_BUS)) != NULL &&
+	    strcmp(str, "bluetooth") == 0)
+		return;
 
-			switch (p->id) {
-			case SPA_PARAM_EnumProfile:
-				break;
-			case SPA_PARAM_Profile:
-				handle_profile(dev, p);
-				break;
-			default:
-				break;
-			}
-		}
-	}
+	if (dev->obj->obj.changed & SM_DEVICE_CHANGE_MASK_PARAMS)
+		handle_profile(dev);
 }
 
 static const struct sm_object_events object_events = {
@@ -258,7 +381,10 @@ static void session_create(void *data, struct sm_object *object)
 	dev->obj = (struct sm_device*)object;
 	dev->id = object->id;
 	dev->impl = impl;
+	dev->name = strdup(name);
 	dev->key = spa_aprintf(PREFIX"%s", name);
+	dev->active_profile = SPA_ID_INVALID;
+	dev->saved_profile = SPA_ID_INVALID;
 
 	dev->obj->obj.mask |= SM_DEVICE_CHANGE_MASK_PARAMS;
 	sm_object_add_listener(&dev->obj->obj, &dev->listener, &object_events, dev);
@@ -267,6 +393,7 @@ static void session_create(void *data, struct sm_object *object)
 static void destroy_device(struct impl *impl, struct device *dev)
 {
 	spa_hook_remove(&dev->listener);
+	free(dev->name);
 	free(dev->key);
 	sm_object_remove_data((struct sm_object*)dev->obj, SESSION_KEY);
 }
