@@ -417,4 +417,6 @@ static uint32_t port_type_value(const char *port_type)
 
 #define METADATA_DEFAULT_SINK           "default.audio.sink"
 #define METADATA_DEFAULT_SOURCE         "default.audio.source"
+#define METADATA_CONFIG_DEFAULT_SINK    "default.configured.audio.sink"
+#define METADATA_CONFIG_DEFAULT_SOURCE  "default.configured.audio.source"
 #define METADATA_TARGET_NODE            "target.node"
