@@ -505,7 +505,7 @@ static void idle_func(struct spa_source *source)
 	if ((r = pw_rtkit_make_realtime(impl->system_bus, 0, rtprio)) < 0) {
 		pw_log_warn("could not make thread realtime: %s", spa_strerror(r));
 	} else {
-		pw_log_info("processing thread made realtime");
+		pw_log_info("processing thread made realtime prio:%d", rtprio);
 	}
 exit:
 	pw_rtkit_bus_free(impl->system_bus);
@@ -546,6 +546,8 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	struct spa_system *system;
 	const struct spa_support *support;
 	uint32_t n_support;
+	const struct pw_properties *props;
+	const char *str;
 	int res;
 
 	support = pw_context_get_support(context, &n_support);
@@ -558,6 +560,11 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
         if (system == NULL)
                 return -ENOTSUP;
 
+	if ((props = pw_context_get_properties(context)) != NULL &&
+	    (str = pw_properties_get(props, "support.dbus")) != NULL &&
+	    !pw_properties_parse_bool(str))
+		return -ENOTSUP;
+
 	impl = calloc(1, sizeof(struct impl));
 	if (impl == NULL)
 		return -ENOMEM;
