@@ -344,7 +344,6 @@ static void emit_params(struct pw_impl_port *port, uint32_t *changed_ids, uint32
 
 static void update_info(struct pw_impl_port *port, const struct spa_port_info *info)
 {
-	struct impl *impl = SPA_CONTAINER_OF(port, struct impl, this);
 	uint32_t changed_ids[MAX_PARAMS], n_changed_ids = 0;
 
 	pw_log_debug(NAME" %p: flags:%08"PRIx64" change_mask:%08"PRIx64,
@@ -378,7 +377,6 @@ static void update_info(struct pw_impl_port *port, const struct spa_port_info *i
 				continue;
 
 			pw_log_debug(NAME" %p: update param %d", port, id);
-			pw_param_clear(&impl->pending_list, id);
 			port->info.params[i] = info->params[i];
 			port->info.params[i].user = 0;
 
@@ -426,6 +424,7 @@ struct pw_impl_port *pw_context_create_port(
 		res = -errno;
 		goto error_no_mem;
 	}
+	pw_properties_setf(properties, PW_KEY_PORT_ID, "%u", port_id);
 
 	if (info) {
 		if (SPA_FLAG_IS_SET(info->flags, SPA_PORT_FLAG_PHYSICAL))
@@ -832,6 +831,7 @@ int pw_impl_port_register(struct pw_impl_port *port,
 		PW_KEY_FORMAT_DSP,
 		PW_KEY_NODE_ID,
 		PW_KEY_AUDIO_CHANNEL,
+		PW_KEY_PORT_ID,
 		PW_KEY_PORT_NAME,
 		PW_KEY_PORT_DIRECTION,
 		PW_KEY_PORT_MONITOR,
@@ -839,6 +839,7 @@ int pw_impl_port_register(struct pw_impl_port *port,
 		PW_KEY_PORT_TERMINAL,
 		PW_KEY_PORT_CONTROL,
 		PW_KEY_PORT_ALIAS,
+		PW_KEY_PORT_EXTRA,
 		NULL
 	};
 
@@ -1069,6 +1070,7 @@ struct result_port_params_data {
 			uint32_t id, uint32_t index, uint32_t next,
 			struct spa_pod *param);
 	int seq;
+	uint32_t count;
 	unsigned int cache:1;
 };
 
@@ -1082,8 +1084,11 @@ static void result_port_params(void *data, int seq, int res, uint32_t type, cons
 		const struct spa_result_node_params *r = result;
 		if (d->seq == seq) {
 			d->callback(d->data, seq, r->id, r->index, r->next, r->param);
-			if (d->cache)
+			if (d->cache) {
+				if (d->count++ == 0)
+					pw_param_add(&impl->pending_list, r->id, NULL);
 				pw_param_add(&impl->pending_list, r->id, r->param);
+			}
 		}
 		break;
 	}
@@ -1105,7 +1110,7 @@ int pw_impl_port_for_each_param(struct pw_impl_port *port,
 	int res;
 	struct impl *impl = SPA_CONTAINER_OF(port, struct impl, this);
 	struct pw_impl_node *node = port->node;
-	struct result_port_params_data user_data = { impl, data, callback, seq, false };
+	struct result_port_params_data user_data = { impl, data, callback, seq, 0, false };
 	struct spa_hook listener;
 	struct spa_param_info *pi;
 	static const struct spa_node_events node_events = {
@@ -1154,7 +1159,8 @@ int pw_impl_port_for_each_param(struct pw_impl_port *port,
 		}
 		res = 0;
 	} else {
-		user_data.cache = filter == NULL;
+		user_data.cache = impl->cache_params &&
+			(filter == NULL && index == 0 && max == UINT32_MAX);
 
 		spa_zero(listener);
 		spa_node_add_listener(node->node, &listener, &node_events, &user_data);
