@@ -122,29 +122,6 @@ static void core_sync(struct data *d)
 	pw_log_debug("sync start %u", d->sync_seq);
 }
 
-static struct param *add_param(struct spa_list *params, uint32_t id, const struct spa_pod *param)
-{
-	struct param *p;
-
-	if (param == NULL || !spa_pod_is_object(param)) {
-		errno = EINVAL;
-		return NULL;
-	}
-	if (id == SPA_ID_INVALID)
-		id = SPA_POD_OBJECT_ID(param);
-
-	p = malloc(sizeof(*p) + SPA_POD_SIZE(param));
-	if (p == NULL)
-		return NULL;
-
-	p->id = id;
-	p->param = SPA_MEMBER(p, sizeof(*p), struct spa_pod);
-	memcpy(p->param, param, SPA_POD_SIZE(param));
-	spa_list_append(params, &p->link);
-
-	return p;
-}
-
 static uint32_t clear_params(struct spa_list *param_list, uint32_t id)
 {
 	struct param *p, *t;
@@ -160,6 +137,35 @@ static uint32_t clear_params(struct spa_list *param_list, uint32_t id)
 	return count;
 }
 
+static struct param *add_param(struct spa_list *params, uint32_t id, const struct spa_pod *param)
+{
+	struct param *p;
+
+	if (id == SPA_ID_INVALID) {
+		if (param == NULL || !spa_pod_is_object(param)) {
+			errno = EINVAL;
+			return NULL;
+		}
+		id = SPA_POD_OBJECT_ID(param);
+	}
+
+	p = malloc(sizeof(*p) + (param != NULL ? SPA_POD_SIZE(param) : 0));
+	if (p == NULL)
+		return NULL;
+
+	p->id = id;
+	if (param != NULL) {
+		p->param = SPA_MEMBER(p, sizeof(*p), struct spa_pod);
+		memcpy(p->param, param, SPA_POD_SIZE(param));
+	} else {
+		clear_params(params, id);
+		p->param = NULL;
+	}
+	spa_list_append(params, &p->link);
+
+	return p;
+}
+
 static struct object *find_object(struct data *d, uint32_t id)
 {
 	struct object *o;
@@ -174,12 +180,14 @@ static void object_update_params(struct object *o)
 {
 	struct param *p;
 
-	spa_list_for_each(p, &o->pending_list, link)
-		clear_params(&o->param_list, p->id);
-
 	spa_list_consume(p, &o->pending_list, link) {
 		spa_list_remove(&p->link);
-		spa_list_append(&o->param_list, &p->link);
+		if (p->param == NULL) {
+			clear_params(&o->param_list, p->id);
+			free(p);
+		} else {
+			spa_list_append(&o->param_list, &p->link);
+		}
 	}
 }
 
@@ -522,7 +530,6 @@ static void core_dump(struct object *o)
 	put_value(d, "host-name", i->host_name);
 	put_value(d, "version", i->version);
 	put_value(d, "name", i->name);
-	put_value(d, "name", i->name);
 	put_flags(d, "change-mask", i->change_mask, fl);
 	put_dict(d, "props", i->props);
 	put_end(d, "}", 0);
@@ -834,7 +841,7 @@ static void node_event_info(void *object, const struct pw_node_info *info)
 			info->params[i].user = 0;
 
 			changed++;
-			clear_params(&o->pending_list, id);
+			add_param(&o->pending_list, id, NULL);
 			if (!(info->params[i].flags & SPA_PARAM_INFO_READ))
 				continue;
 
@@ -917,7 +924,7 @@ static void port_event_info(void *object, const struct pw_port_info *info)
 			info->params[i].user = 0;
 
 			changed++;
-			clear_params(&o->pending_list, id);
+			add_param(&o->pending_list, id, NULL);
 			if (!(info->params[i].flags & SPA_PARAM_INFO_READ))
 				continue;
 
@@ -1347,7 +1354,8 @@ static void show_help(struct data *data, const char *name)
 		"  -h, --help                            Show this help\n"
 		"      --version                         Show version\n"
 		"  -r, --remote                          Remote daemon name\n"
-		"  -m, --monitor                         monitor changes\n",
+		"  -m, --monitor                         monitor changes\n"
+		"  -N, --no-colors                       disable color output\n",
 		name);
 }
 
@@ -1362,13 +1370,17 @@ int main(int argc, char *argv[])
 		{ "version",	no_argument,		NULL, 'V' },
 		{ "remote",	required_argument,	NULL, 'r' },
 		{ "monitor",	no_argument,		NULL, 'm' },
+		{ "no-colors",	no_argument,		NULL, 'N' },
 		{ NULL, 0, NULL, 0}
 	};
 	int c;
 
 	pw_init(&argc, &argv);
 
-	while ((c = getopt_long(argc, argv, "hVr:m", long_options, NULL)) != -1) {
+	data.out = stdout;
+	colors = true;
+
+	while ((c = getopt_long(argc, argv, "hVr:mN", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'h' :
 			show_help(&data, argv[0]);
@@ -1387,6 +1399,9 @@ int main(int argc, char *argv[])
 		case 'm' :
 			data.monitor = true;
 			break;
+		case 'N' :
+			colors = false;
+			break;
 		default:
 			show_help(&data, argv[0]);
 			return -1;
@@ -1407,10 +1422,6 @@ int main(int argc, char *argv[])
 	pw_loop_add_signal(l, SIGINT, do_quit, &data);
 	pw_loop_add_signal(l, SIGTERM, do_quit, &data);
 
-	data.out = stdout;
-	if (isatty(fileno(data.out)))
-		colors = true;
-
 	data.context = pw_context_new(l, NULL, 0);
 	if (data.context == NULL) {
 		fprintf(stderr, "can't create context: %m\n");
